\name{arrayspc}
\alias{arrayspc}
\title{Sparse PCs of Microarrays}
\description{Sparse PC by iterative SVD and soft-thresholding}
\usage{
arrayspc(x,K=1,para,use.corr=FALSE, max.iter=100,trace=FALSE,eps=1e-3)
}
\arguments{
\item{x}{
The microarray matrix.
}
\item{K}{
Number of components. Default is 1.
}
\item{para}{The thresholding parameters. A vector of length K. 
}
\item{use.corr}{Perform PCA on the correlation matrix? This option is
  only effective when the argument type is set "data".}
\item{max.iter}{Maximum number of iterations.}
\item{trace}{If TRUE, prints out its progress.}
\item{eps}{Convergence criterion.}
}
\value{A "arrayspc" object is returned.}
\details{
The function is equivalent to a special case of spca() with the quadratic
penalty=infinity. It is specifically designed for the case p>>n, like microarrays.
}\references{
Zou, H., Hastie, T. and Tibshirani, R. (2004) "Sparse principal component
analysis" \emph{Technical report, Statistics Dept. Stanford University}
}
\seealso{
spca, princomp
}
\author{Hui Zou, Trevor Hastie and Robert Tibshirani}
\keyword{multivariate}





\eof
\name{cv.enet}
\alias{cv.enet}
\title{Computes K-fold cross-validated error curve for elastic net}
\description{
Computes the K-fold cross-validated mean squared prediction error for
elastic net.}
\usage{
cv.enet(x, y, K = 10, lambda, s, mode,trace = FALSE, plot.it = TRUE, se = TRUE, ...)
}
\arguments{
  \item{x}{Input to lars}
  \item{y}{Input to lars}
  \item{K}{Number of folds}
  \item{lambda}{Quadratic penalty parameter}
  \item{s}{Abscissa values at which CV curve should be computed.
    A value, or vector of values, indexing the path. Its values depends on the mode= argument}
\item{mode}{Mode="step" means the s= argument indexes the LARS-EN step number. If mode="fraction", then s should be a number
between 0 and 1, and it refers to the ratio of the L1 norm of the
coefficient vector, relative to the norm at the full LS solution.
Mode="norm" means s refers to the L1 norm of the coefficient vector.
Abbreviations allowed. If mode="norm", then s should be the L1 norm of
the coefficient vector. If mode="penalty", then s should be the 1-norm
penalty parameter.}
  \item{trace}{Show computations?}
  \item{plot.it}{Plot it?}
  \item{se}{Include standard error bands?}
  \item{\dots}{Additional arguments to \code{enet}}
}
\value{
Invisibly returns a list with components (which can be plotted using \code{plotCVLars})
  \item{fraction}{Values of s}
  \item{cv}{The CV curve at each value of fraction}
  \item{cv.error}{The standard error of the CV curve}
}
\references{
Zou and Hastie (2004) "Regularization and
Variable Selection via the Elastic Net"
\emph{In press, Journal of the Royal Statistical Society, Series B}.
}
\author{Hui Zou and Trevor Hastie}
\examples{
data(diabetes)
attach(diabetes)
## use the L1 fraction norm as the tuning parameter
cv.enet(x2,y,lambda=0.05,s=seq(0,1,length=100),mode="fraction",trace=TRUE,max.steps=80)
## use the number of steps as the tuning parameter
cv.enet(x2,y,lambda=0.05,s=1:50,mode="step")
detach(diabetes)
}
\keyword{regression}


\eof
\name{diabetes}
\alias{diabetes}
\title{Blood and other measurements in diabetics}
\description{
 The \code{diabetes} data frame has 442 rows and 3 columns.
These are the data used in the Efron et al "Least Angle Regression" paper.
}
\format{
  This data frame contains the following columns:
  \describe{
    \item{x}{a matrix with 10 columns}
    \item{y}{a numeric vector}
    \item{x2}{a matrix with 64 columns}
  }
}
\details{
The x matrix has been standardized to have unit L2 norm in each column
and zero mean. The matrix x2 consists of x plus certain interactions.
}
\source{
  \url{http://www-stat.stanford.edu/~hastie/Papers/LARS/LeastAngle_2002.ps}
  }
\references{
Efron, Hastie, Johnstone and Tibshirani (2003) "Least Angle Regression"
(with discussion) \emph{Annals of Statistics}
}
\keyword{datasets}

\eof
\name{elasticnet-internal}
\title{Internal elasticnet functions} 
\alias{updateRR}
\alias{solvebeta}
\alias{rootmatrix}
\alias{soft}
\description{Internal elasticnet functions}
\usage{
updateRR(xnew, R = NULL, xold, lambda, eps = .Machine$double.eps)
solvebeta(x, y, paras, max.steps, sparse=c("penalty","varnum"), eps = .Machine$double.eps)
rootmatrix(x)
soft(a,para)
}
\author{Hui Zou and Trevor Hastie}
\details{
  These are not to be called by the user.
}
\keyword{internal}

\eof
\name{enet}
\alias{enet}
\title{
Fits Elastic Net regression models
}
\description{
  Starting from zero, the LARS-EN algorithm provides the entire sequence of coefficients and fits.
}
\usage{
enet(x, y, lambda, max.steps, normalize=TRUE, intercept=TRUE, trace = FALSE, eps = .Machine$double.eps)
}
\arguments{
\item{x}{
matrix of predictors
}
\item{y}{
response 
}
\item{lambda}{
Quadratic penalty parameter. lambda=0 performs the Lasso fit.
}
\item{max.steps}{
  Limit the number of steps taken; the default is \code{50 * min(m,
    n-1)}, with m the number of variables, and n the number of samples.
One can use this option to perform early stopping.
}
\item{trace}{
If TRUE, prints out its progress
}
\item{normalize}{ Standardize the predictors?
}
\item{intercept}{ Center the predictors?
}
\item{eps}{
An effective zero
}}
\value{
An "enet" object is returned, for which print, plot and predict methods exist.
}
\details{
The Elastic Net methodology is described in detail in Zou and Hastie (2004).
The LARS-EN algorithm computes the complete elastic net
solution simultaneously for ALL values of the shrinkage parameter in
the same computational cost as a least squares fit. 
The structure of enet() is based on lars() coded by Efron and Hastie.  
Some internel functions from the lars package are called. 
The user should install lars before using elasticnet functions.
}
\references{
Zou and Hastie (2004) "Regularization and
Variable Selection via the Elastic Net"
\emph{In press, Journal of the Royal Statistical Society, Series B}.
}
\author{Hui Zou and Trevor Hastie}
\seealso{
print, plot, and predict methods for enet
}
\examples{
data(diabetes)
attach(diabetes)
##fit the lasso model (treated as a special case of the elastic net)
object1 <- enet(x,y,lambda=0)
plot(object1)
##fit the elastic net model with lambda=1.
object2 <- enet(x,y,lambda=1) 
plot(object2)
##early stopping after 50 LARS-EN steps
object4 <- enet(x2,y,lambda=0.5,max.steps=50)
plot(object4)
detach(diabetes)
}
\keyword{regression}




\eof
\name{pitprops}
\alias{pitprops}
\title{Pitprops correlation data}
\description{
The \code{pitprops} data is a correlation matrix that was calculated from 180 observations. There are 13 
explanatory variables.
}
\usage{
data(pitprops)
}
\details{
Jeffers (1967) tried to interpret the first six PCs. This is a classical example showing the
difficulty of interpreting principal components.
}
\references{
Jeffers, J. (1967) "Two case studies in the application of principal component",    
\emph{Applied Statistics}, 16, 225-236.}
\keyword{datasets}

\eof
\name{plot.enet}
\alias{plot.enet}
\title{Plot method for enet objects}
\description{
Produce a plot of an enet fit. The default is a complete coefficient path.
}
\usage{
plot.enet(x, xvar = c("fraction", "penalty", "L1norm", "step"),
use.color = FALSE, ...) 
}
\arguments{
  \item{x}{enet object}
  \item{xvar}{The type of x variable against which to
plot. \code{xvar=fraction} plots agains the fraction of the L1 norm of the coefficient vector (default).
\code{xvar=penalty} plots against the 1-norm penalty parameter.
\code{xvar=L1norm} plots against the L1 norm of the coefficient
vector. \code{xvar=step} plots against the LARS-EN step number. 
}
\item{use.color}{a colorful plot?}
\item{\dots}{Additonal arguments for generic plot.}
}
\value{NULL}
\references{
Zou and Hastie (2004) "Regularization and Variable Selection via the Elastic Net"
\emph{In press, Journal of the Royal Statistical Society, Series B}.
}
\author{Hui Zou and Trevor Hastie}
\examples{
data(diabetes)
attach(diabetes)
object <- enet(x,y,lambda=1)
par(mfrow=c(2,2))
plot(object)
plot(object,xvar="step")
detach(diabetes)
}
\keyword{hplot}
\keyword{methods}

\eof
\name{predict.enet}
\alias{predict.enet}
\title{
Make predictions or extract coefficients from a fitted elastic net model
}
\description{
While enet() produces the entire path of solutions, predict.enet
allows one to extract a prediction at a particular point along the path.
}
\usage{
predict.enet(object, newx, s, type = c("fit", "coefficients"), mode =
c("step","fraction", "norm", "penalty"),naive=FALSE, ...)
}
\arguments{
\item{object}{
A fitted enet object
}
\item{newx}{
If type="fit", then newx should be the x values at which the fit is
required. If type="coefficients", then newx can be omitted.
}
\item{s}{
a value, or vector of values, indexing the path. Its values depends on the mode= argument. By
default (mode="step").
}
\item{type}{
If type="fit", predict returns the fitted values. If
type="coefficients", predict returns the coefficients.
Abbreviations allowed.
}
\item{mode}{
Mode="step" means the s= argument indexes the LARS-EN step number, and
the coefficients will be returned corresponding to the values
corresponding to step s. If mode="fraction", then s should be a number
between 0 and 1, and it refers to the ratio of the L1 norm of the
coefficient vector, relative to the norm at the full LS solution.
Mode="norm" means s refers to the L1 norm of the coefficient vector.
Abbreviations allowed. If mode="norm", then s should be the L1 norm of
the coefficient vector. If mode="penalty", then s should be the 1-norm
penalty parameter.
}
\item{naive}{IF naive is True, then the naive elastic net fit is
  returned.}
\item{\dots}{Additonal arguments for generic print.}
}
\value{
Either a vector/matrix of fitted values, or a vector/matrix of coefficients.
}
\details{
Starting from zero, the LARS-EN algorithm provides the entire sequence of coefficients and fits.
}\references{
Zou and Hastie (2004) "Regularization and
Variable Selection via the Elastic Net"
\emph{In press, Journal of the Royal Statistical Society, Series B}.
}
\author{Hui Zou and Trevor Hastie}
\seealso{
print, plot,  enet
}
\examples{
data(diabetes)
attach(diabetes)
object <- enet(x,y,lambda=0.1)
### make predictions at the values in x, at each of the
### steps produced in object
fits <- predict.enet(object, x, type="fit")
### extract the coefficient vector with L1 norm=2000
coef2000 <- predict(object, s=2000, type="coef", mode="norm")
### extract the coefficient vector with L1 norm fraction=0.45
coef.45 <- predict(object, s=0.45, type="coef", mode="fraction")
detach(diabetes)
}
\keyword{regression}
\keyword{methods}


\eof
\name{print.arrayspc}
\alias{print.arrayspc}
\title{Print method for arrayspc objects}
\description{
Print out an arrayspc fit. 
}
\usage{
print.arrayspc<-function(x, ...)  
}
\arguments{
  \item{x}{arrayspc object}
\item{\dots}{Additonal arguments for generic print.}
}
\value{NULL}
\references{
Zou, H., Hastie, T. and Tibshirani, R. (2004) "Sparse principal component
analysis" \emph{Technical report, Statistics Dept. Stanford University}.
}
\author{Hui Zou and Trevor Hastie}
\keyword{methods}


\eof
\name{print.enet}
\alias{print.enet}
\title{Print method for enet objects}
\description{
Print out an enet fit. 
}
\usage{
print.enet(x, ...)  
}
\arguments{
  \item{x}{enet object}
\item{\dots}{Additonal arguments for generic print.}
}
\value{NULL}
\references{
Zou and Hastie (2004) "Regularization and Variable Selection via the Elastic Net"
\emph{In press, Journal of the Royal Statistical Society, Series B}.
}
\author{Hui Zou and Trevor Hastie}
\keyword{methods}

\eof
\name{print.spca}
\alias{print.spca}
\title{Print method for spca objects}
\description{
Print out a spca fit. 
}
\usage{
print.spca(x, ...)  
}
\arguments{
  \item{x}{spca object}
\item{\dots}{Additonal arguments for generic print.}
}
\value{NULL}
\references{
Zou, H., Hastie, T. and Tibshirani, R. (2004) "Sparse principal component
analysis" \emph{Technical report, Statistics Dept. Stanford University}.
}
\author{Hui Zou and Trevor Hastie}
\keyword{methods}

\eof
\name{spca}
\alias{spca}
\title{Sparse Principal Components Analysis}
\description{Using an alternating minimization algorithm to minimize the SPCA criterion.}
\usage{
spca(x,K,para,type=c("predictor","Gram"),sparse=c("penalty","varnum"),use.corr=FALSE,lambda=1e-6,max.iter=200,trace=FALSE,eps.conv=1e-3)
}
\arguments{
\item{x}{
A matrix. It can be the predictor matrix or the sample
covariance/correlation matrix. 
}
\item{K}{
Number of components
}
\item{para}{A vector of length K. All elements should be positive. If sparse="varnum", the elements integers.
}
\item{type}{
If type="predictor", x is the predictor matrix.
If type="Gram", the function asks the user to provide the sample covariance or correlation matrix.  
}
\item{sparse}{If sparse="penalty", para is a vector of 1-norm
  penalty parameters. If sparse="varnum", para defines the number of
  sparse loadings to be obtained. This option is not discussed in the
  paper given below, but it is convenient in practice.}
\item{lambda}{Quadratic penalty parameter. Default value is 1e-6.}
\item{use.corr}{Perform PCA on the correlation matrix? This option is
  only effective when the argument type is set "data".}
\item{max.iter}{Maximum number of iterations.}
\item{trace}{If TRUE, prints out its progress.}
\item{eps.conv}{Convergence criterion.}
}
\value{A "spca" object is returned. 
The below are some quantities which the user may be interested in: 
\item{loadings}{
The loadings of the sparse PCs
}
\item{pev}{ 
Percentage of explained variance
}
\item{var.all}{ 
Total variance of the predictors
}
}
\details{
PCA is shown to be equivalent to a regression-type optimization problem,
then sparse loadings are obtained by imposing the 1-norm constraint on
the regression coefficients. If x is a microarray matrix, use arrayspc().
}\references{
Zou, H., Hastie, T. and Tibshirani, R. (2004) "Sparse principal component
analysis" \emph{Technical report, Statistics Dept. Stanford University}.
}
\author{Hui Zou, Trevor Hastie and Robert Tibshirani}
\seealso{
princomp, arrayspc
}
\examples{
data(pitprops)
out1<-spca(pitprops,K=6,type="Gram",sparse="penalty",trace=TRUE,para=c(0.06,0.16,0.1,0.5,0.5,0.5))
## print the object out1
out1
out2<-spca(pitprops,K=6,type="Gram",sparse="varnum",trace=TRUE,para=c(7,4,4,1,1,1))
out2
## to see the contents of out2
names(out2) 
## to get the loadings
out2$loadings
}
\keyword{multivariate}


\eof
